==> syd-3.3.2.tar.gz: syd-v3.3.2/src/unshare/mod.rs <==
//! The `Command` has mostly same API as `std::process::Command` except where
//! is absolutely needed.
//!
//! In addition `Command` contains methods to configure linux namespaces,
//! chroots and more linux stuff.
//!
//! We have diverged from ``std::process::Command`` in the following
//! major things:
//!
//! 1. Error handling. Since sometimes we have long chains of system calls
//!    involved, we need to give user some way to find out which call failed
//!    with an error, so `io::Error` is not an option.  We have
//!    ``error::Error`` class which describes the error as precisely as
//!    possible
//!
//! 2. We set ``PDEATHSIG`` to ``SIGKILL`` by default. I.e. child process will
//!    die when parent is dead. This is what you want most of the time. If you
//!    want to allow child process to daemonize explicitly call the
//!    ``allow_daemonize`` method (but look at documentation of
//!    ``Command::set_parent_death_signal`` first).
//!
//! 3. We don't search for `program` in `PATH`. It's hard to do right in all
//!    cases of `chroot`, `pivot_root`, user and mount namespaces. So we expect
//!    its easier to do for your specific container setup.
//!
//! Anyway this is low-level interface. You may want to use some higher level
//! abstraction which mounts filesystems, sets network and monitors processes.
#![warn(missing_docs)]

mod callbacks;
mod caps;
mod child;
mod chroot;
mod config;
mod debug;
mod error;
mod fds;
mod ffi_util;
mod idmap;
mod linux;
mod namespace;
mod pipe;
mod run;
mod seccomp;
mod status;
mod std_api;
mod stdio;
mod wait;
mod zombies;

use std::{collections::HashMap, ffi::CString, io, os::unix::io::RawFd, path::PathBuf};

use libseccomp::ScmpFilterContext;
use nix::libc::pid_t;
pub use nix::sys::signal::Signal;

use crate::unshare::pipe::PipeHolder;
pub use crate::unshare::{
    caps::Capability,
    debug::{Printer, Style},
    error::Error,
    idmap::{GidMap, UidMap},
    namespace::Namespace,
    pipe::{PipeReader, PipeWriter},
    status::ExitStatus,
    stdio::{Fd, Stdio},
    zombies::{child_events, reap_zombies, ChildEvent},
};

type BoxError = Box<dyn (::std::error::Error) + Send + Sync + 'static>;

/// Main class for running processes. Works in the spirit of builder pattern.
#[allow(clippy::type_complexity)]
pub struct Command {
    filename: CString,
    args: Vec<CString>,
    config: config::Config,
    fds: HashMap<RawFd, Fd>,
    close_fds: Vec<(RawFd, RawFd)>,
    chroot_dir: Option<PathBuf>,
    pivot_root: Option<(PathBuf, PathBuf, bool)>,
    id_map_commands: Option<(PathBuf, PathBuf)>,
    before_unfreeze: Option<Box<dyn FnMut(u32) -> Result<(), BoxError>>>,
    pre_exec: Option<Box<dyn Fn() -> Result<(), io::Error>>>,
    seccomp_filter: Option<ScmpFilterContext>,
    seccomp_channel: Option<crate::hook::SocketPair>,
    host_name: Option<String>,
    domain_name: Option<String>,
}

/// The reference to the running child
#[derive(Debug)]
pub struct Child {
    pid: pid_t,
    status: Option<ExitStatus>,
    fds: HashMap<RawFd, PipeHolder>,
    /// Stdin of a child if it is a pipe
    pub stdin: Option<PipeWriter>,
    /// Stdout of a child if it is a pipe
    pub stdout: Option<PipeReader>,
    /// Stderr of a child if it is a pipe
    pub stderr: Option<PipeReader>,
    /// Seccomp file descriptor.
    pub seccomp_fd: Option<RawFd>,
}

==> syd-3.9.5.tar.gz: syd-v3.9.5/src/unshare/mod.rs <==
//! The `Command` has mostly same API as `std::process::Command` except where
//! is absolutely needed.
//!
//! In addition `Command` contains methods to configure linux namespaces,
//! chroots and more linux stuff.
//!
//! We have diverged from ``std::process::Command`` in the following
//! major things:
//!
//! 1. Error handling. Since sometimes we have long chains of system calls
//!    involved, we need to give user some way to find out which call failed
//!    with an error, so `io::Error` is not an option.  We have
//!    ``error::Error`` class which describes the error as precisely as
//!    possible
//!
//! 2. We set ``PDEATHSIG`` to ``SIGKILL`` by default. I.e. child process will
//!    die when parent is dead. This is what you want most of the time. If you
//!    want to allow child process to daemonize explicitly call the
//!    ``allow_daemonize`` method (but look at documentation of
//!    ``Command::set_parent_death_signal`` first).
//!
//! 3. We don't search for `program` in `PATH`. It's hard to do right in all
//!    cases of `chroot`, `pivot_root`, user and mount namespaces. So we expect
//!    its easier to do for your specific container setup.
//!
//! Anyway this is low-level interface. You may want to use some higher level
//! abstraction which mounts filesystems, sets network and monitors processes.
#![warn(missing_docs)]

mod callbacks;
/// Linux capabilities
pub mod caps;
mod child;
mod config;
mod debug;
mod error;
mod ffi_util;
mod idmap;
mod linux;
/// Utilities for Linux namespaces
pub mod namespace;
mod run;
mod seccomp;
mod status;
mod std_api;
mod stdio;
mod wait;
mod zombies;

use std::{
    ffi::CString,
    os::unix::{io::RawFd, net::UnixStream},
};

use libseccomp::ScmpFilterContext;
pub use nix::sys::signal::Signal;
use nix::{errno::Errno, libc::pid_t};

pub use crate::unshare::{
    caps::Capability,
    debug::{Printer, Style},
    error::Error,
    idmap::{GidMap, UidMap},
    namespace::Namespace,
    status::ExitStatus,
    stdio::{Fd, Stdio},
    zombies::{child_events, reap_zombies, ChildEvent},
};

/// Main class for running processes. Works in the spirit of builder pattern.
#[allow(clippy::type_complexity)]
pub struct Command {
    filename: CString,
    args: Vec<CString>,
    config: config::Config,
    before_unfreeze: Option<Box<dyn FnMut(u32) -> Result<(), Errno>>>,
    pre_exec: Option<Box<dyn Fn() -> Result<(), Errno>>>,
    seccomp_filter: Option<ScmpFilterContext>,
    seccomp_channel: Option<(UnixStream, UnixStream)>,
}

/// The reference to the running child
#[derive(Debug)]
pub struct Child {
    pid: pid_t,
    status: Option<ExitStatus>,
    /// Seccomp file descriptor.
    pub seccomp_fd: Option<RawFd>,
}
